class OptionsHolder:
    def __init__(self):
        self.Mesh = Mesh()
        self.General = General()
        self.Geometry = Geometry()


class General:
    def __init__(self):
        self.AlphaBlending = None
        self.Antialiasing = None
        self.ArrowHeadRadius = None
        self.ArrowStemLength = None
        self.ArrowStemRadius = None
        self.Axes = None
        self.AxesAutoPosition = None
        self.AxesForceValue = None
        self.AxesFormatX = None
        self.AxesFormatY = None
        self.AxesFormatZ = None
        self.AxesLabelX = None
        self.AxesLabelY = None
        self.AxesLabelZ = None
        self.AxesMaxX = None
        self.AxesMaxY = None
        self.AxesMaxZ = None
        self.AxesMikado = None
        self.AxesMinX = None
        self.AxesMinY = None
        self.AxesMinZ = None
        self.AxesTicsX = None
        self.AxesTicsY = None
        self.AxesTicsZ = None
        self.AxesValueMaxX = None
        self.AxesValueMaxY = None
        self.AxesValueMaxZ = None
        self.AxesValueMinX = None
        self.AxesValueMinY = None
        self.AxesValueMinZ = None
        self.BackgroundGradient = None
        self.BackgroundImage3D = None
        self.BackgroundImageFileName = None
        self.BackgroundImageHeight = None
        self.BackgroundImagePage = None
        self.BackgroundImagePositionX = None
        self.BackgroundImagePositionY = None
        self.BackgroundImageWidth = None
        self.BoundingBoxSize = None
        self.Camera = None
        self.CameraAperture = None
        self.CameraEyeSeparationRatio = None
        self.CameraFocalLengthRatio = None
        self.Clip0A = None
        self.Clip0B = None
        self.Clip0C = None
        self.Clip0D = None
        self.Clip1A = None
        self.Clip1B = None
        self.Clip1C = None
        self.Clip1D = None
        self.Clip2A = None
        self.Clip2B = None
        self.Clip2C = None
        self.Clip2D = None
        self.Clip3A = None
        self.Clip3B = None
        self.Clip3C = None
        self.Clip3D = None
        self.Clip4A = None
        self.Clip4B = None
        self.Clip4C = None
        self.Clip4D = None
        self.Clip5A = None
        self.Clip5B = None
        self.Clip5C = None
        self.Clip5D = None
        self.ClipFactor = None
        self.ClipOnlyDrawIntersectingVolume = None
        self.ClipOnlyVolume = None
        self.ClipPositionX = None
        self.ClipPositionY = None
        self.ClipWholeElements = None
        self.Color = ColorGeneral()
        self.ColorScheme = None
        self.ConfirmOverwrite = None
        self.ContextPositionX = None
        self.ContextPositionY = None
        self.DefaultFileName = None
        self.DetachedMenu = None
        self.Display = None
        self.DisplayBorderFactor = None
        self.DoubleBuffer = None
        self.DrawBoundingBoxes = None
        self.ErrorFileName = None
        self.ExecutableFileName = None
        self.ExpertMode = None
        self.ExtraHeight = None
        self.ExtraPositionX = None
        self.ExtraPositionY = None
        self.ExtraWidth = None
        self.FastRedraw = None
        self.FieldHeight = None
        self.FieldPositionX = None
        self.FieldPositionY = None
        self.FieldWidth = None
        self.FileChooserPositionX = None
        self.FileChooserPositionY = None
        self.FileName = None
        self.FltkColorScheme = None
        self.FltkTheme = None
        self.FontSize = None
        self.GraphicsFont = None
        self.GraphicsFontEngine = None
        self.GraphicsFontSize = None
        self.GraphicsFontSizeTitle = None
        self.GraphicsFontTitle = None
        self.GraphicsHeight = None
        self.GraphicsPositionX = None
        self.GraphicsPositionY = None
        self.GraphicsWidth = None
        self.HighOrderToolsPositionX = None
        self.HighOrderToolsPositionY = None
        self.HighResolutionGraphics = None
        self.HighResolutionPointSizeFactor = None
        self.InitialModule = None
        self.Light0 = None
        self.Light0W = None
        self.Light0X = None
        self.Light0Y = None
        self.Light0Z = None
        self.Light1 = None
        self.Light1W = None
        self.Light1X = None
        self.Light1Y = None
        self.Light1Z = None
        self.Light2 = None
        self.Light2W = None
        self.Light2X = None
        self.Light2Y = None
        self.Light2Z = None
        self.Light3 = None
        self.Light3W = None
        self.Light3X = None
        self.Light3Y = None
        self.Light3Z = None
        self.Light4 = None
        self.Light4W = None
        self.Light4X = None
        self.Light4Y = None
        self.Light4Z = None
        self.Light5 = None
        self.Light5W = None
        self.Light5X = None
        self.Light5Y = None
        self.Light5Z = None
        self.LineWidth = None
        self.ManipulatorPositionX = None
        self.ManipulatorPositionY = None
        self.MaxX = None
        self.MaxY = None
        self.MaxZ = None
        self.MenuHeight = None
        self.MenuPositionX = None
        self.MenuPositionY = None
        self.MenuWidth = None
        self.MeshDiscrete = None
        self.MessageFontSize = None
        self.MessageHeight = None
        self.MinX = None
        self.MinY = None
        self.MinZ = None
        self.MouseHoverMeshes = None
        self.MouseSelection = None
        self.NonModalWindows = None
        self.NoPopup = None
        self.OptionsFileName = None
        self.OptionsPositionX = None
        self.OptionsPositionY = None
        self.Orthographic = None
        self.PluginHeight = None
        self.PluginPositionX = None
        self.PluginPositionY = None
        self.PluginWidth = None
        self.PointSize = None
        self.PolygonOffsetAlwaysOn = None
        self.PolygonOffsetFactor = None
        self.PolygonOffsetUnits = None
        self.ProgressMeterStep = None
        self.QuadricSubdivisions = None
        self.RecentFile0 = None
        self.RecentFile1 = None
        self.RecentFile2 = None
        self.RecentFile3 = None
        self.RecentFile4 = None
        self.RecentFile5 = None
        self.RecentFile6 = None
        self.RecentFile7 = None
        self.RecentFile8 = None
        self.RecentFile9 = None
        self.RotationCenterGravity = None
        self.RotationCenterX = None
        self.RotationCenterY = None
        self.RotationCenterZ = None
        self.RotationX = None
        self.RotationY = None
        self.RotationZ = None
        self.SaveOptions = None
        self.SaveSession = None
        self.ScaleX = None
        self.ScaleY = None
        self.ScaleZ = None
        self.SessionFileName = None
        self.Shininess = None
        self.ShininessExponent = None
        self.SmallAxes = None
        self.SmallAxesPositionX = None
        self.SmallAxesPositionY = None
        self.SmallAxesSize = None
        self.StatisticsPositionX = None
        self.StatisticsPositionY = None
        self.Stereo = None
        self.SystemMenuBar = None
        self.Terminal = None
        self.TextEditor = None
        self.TmpFileName = None
        self.Tooltips = None
        self.Trackball = None
        self.TrackballHyperbolicSheet = None
        self.TrackballQuaternion0 = None
        self.TrackballQuaternion1 = None
        self.TrackballQuaternion2 = None
        self.TrackballQuaternion3 = None
        self.TranslationX = None
        self.TranslationY = None
        self.TranslationZ = None
        self.VectorType = None
        self.Verbosity = None
        self.VisibilityPositionX = None
        self.VisibilityPositionY = None
        self.WatchFilePattern = None
        self.ZoomFac = None

class Geometry:
    def __init__(self):
        self.AutoCoherence = None
        self.Clip = None
        self.Color = ColorGeometry()
        self.CopyDisplayAttributes = None
        self.CopyMeshingMethod = None
        self.DoubleClickedEntityTag = None
        self.DoubleClickedLineCommand = None
        self.DoubleClickedPointCommand = None
        self.DoubleClickedSurfaceCommand = None
        self.DoubleClickedVolumeCommand = None
        self.ExactExtrusion = None
        self.ExtrudeReturnLateralEntities = None
        self.ExtrudeSplinePoints = None
        self.HideCompounds = None
        self.HighlightOrphans = None
        self.LabelType = None
        self.Light = None
        self.LightTwoSide = None
        self.LineNumbers = None
        self.Lines = None
        self.LineSelectWidth = None
        self.LineType = None
        self.LineWidth = None
        self.MatchGeomAndMesh = None
        self.Normals = None
        self.NumSubEdges = None
        self.OCCConnectFaces = None
        self.OCCFixDegenerated = None
        self.OCCFixSmallEdges = None
        self.OCCFixSmallFaces = None
        self.OCCScaling = None
        self.OCCSewFaces = None
        self.OffsetX = None
        self.OffsetY = None
        self.OffsetZ = None
        self.OldCircle = None
        self.OldNewReg = None
        self.OldRuledSurface = None
        self.OrientedPhysicals = None
        self.PointNumbers = None
        self.Points = None
        self.PointSelectSize = None
        self.PointSize = None
        self.PointType = None
        self.ScalingFactor = None
        self.SnapX = None
        self.SnapY = None
        self.SnapZ = None
        self.SurfaceNumbers = None
        self.Surfaces = None
        self.SurfaceType = None
        self.Tangents = None
        self.Tolerance = None
        self.Transform = None
        self.TransformXX = None
        self.TransformXY = None
        self.TransformXZ = None
        self.TransformYX = None
        self.TransformYY = None
        self.TransformYZ = None
        self.TransformZX = None
        self.TransformZY = None
        self.TransformZZ = None
        self.VolumeNumbers = None
        self.Volumes = None

class Mesh:
    def __init__(self):
        self.Algorithm = None
        self.Algorithm3D = None
        self.AllowSwapAngle = None
        self.AngleSmoothNormals = None
        self.AnisoMax = None
        self.BdfFieldFormat = None
        self.Binary = None
        self.CgnsImportOrder = None
        self.ChacoArchitecture = None
        self.ChacoEigensolver = None
        self.ChacoEigTol = None
        self.ChacoGlobalMethod = None
        self.ChacoHypercubeDim = None
        self.ChacoLocalMethod = None
        self.ChacoMeshDim1 = None
        self.ChacoMeshDim2 = None
        self.ChacoMeshDim3 = None
        self.ChacoParamINTERNAL_VERTICES = None
        self.ChacoParamREFINE_MAP = None
        self.ChacoParamREFINE_PARTITION = None
        self.ChacoParamTERMINAL_PROPOGATION = None
        self.ChacoPartitionSection = None
        self.ChacoSeed = None
        self.ChacoVMax = None
        self.CharacteristicLengthExtendFromBoundary = None
        self.CharacteristicLengthFactor = None
        self.CharacteristicLengthFromCurvature = None
        self.CharacteristicLengthFromPoints = None
        self.CharacteristicLengthMax = None
        self.CharacteristicLengthMin = None
        self.Clip = None
        self.Color = ColorMesh()
        self.ColorCarousel = None
        self.CpuTime = None
        self.DoRecombinationTest = None
        self.DrawSkinOnly = None
        self.Dual = None
        self.ElementOrder = None
        self.Explode = None
        self.FlexibleTransfinite = None
        self.Format = None
        self.Hexahedra = None
        self.HighOrderNumLayers = None
        self.HighOrderOptimize = None
        self.HighOrderOptPrimSurfMesh = None
        self.HighOrderPoissonRatio = None
        self.HighOrderThresholdMax = None
        self.HighOrderThresholdMin = None
        self.IgnorePartitionBoundary = None
        self.LabelSampling = None
        self.LabelType = None
        self.LcIntegrationPrecision = None
        self.Light = None
        self.LightLines = None
        self.LightTwoSide = None
        self.LineNumbers = None
        self.Lines = None
        self.LineWidth = None
        self.Lloyd = None
        self.MeshOnlyVisible = None
        self.MetisAlgorithm = None
        self.MetisEdgeMatching = None
        self.MetisRefinementAlgorithm = None
        self.MinimumCirclePoints = None
        self.MinimumCurvePoints = None
        self.MshFilePartitioned = None
        self.MshFileVersion = None
        self.NbHexahedra = None
        self.NbNodes = None
        self.NbPartitions = None
        self.NbPrisms = None
        self.NbPyramids = None
        self.NbQuadrangles = None
        self.NbTetrahedra = None
        self.NbTriangles = None
        self.NbTrihedra = None
        self.NewtonConvergenceTestXYZ = None
        self.Normals = None
        self.NumSubEdges = None
        self.OldRefinement = None
        self.Optimize = None
        self.OptimizeNetgen = None
        self.Partitioner = None
        self.PartitionHexWeight = None
        self.PartitionPrismWeight = None
        self.PartitionPyramidWeight = None
        self.PartitionQuadWeight = None
        self.PartitionTetWeight = None
        self.PartitionTrihedronWeight = None
        self.PartitionTriWeight = None
        self.PointNumbers = None
        self.Points = None
        self.PointSize = None
        self.PointType = None
        self.PreserveNumberingMsh2 = None
        self.Prisms = None
        self.Pyramids = None
        self.Quadrangles = None
        self.QualityInf = None
        self.QualitySup = None
        self.QualityType = None
        self.RadiusInf = None
        self.RadiusSup = None
        self.RandomFactor = None
        self.RecombinationAlgorithm = None
        self.RecombinationTestHorizStart = None
        self.RecombinationTestNoGreedyStrat = None
        self.Recombine3DAll = None
        self.Recombine3DConformity = None
        self.Recombine3DLevel = None
        self.RecombineAll = None
        self.RefineSteps = None
        self.RemeshAlgorithm = None
        self.RemeshParametrization = None
        self.SaveAll = None
        self.SaveElementTagType = None
        self.SaveGroupsOfNodes = None
        self.SaveParametric = None
        self.ScalingFactor = None
        self.SecondOrderExperimental = None
        self.SecondOrderIncomplete = None
        self.SecondOrderLinear = None
        self.SmoothCrossField = None
        self.Smoothing = None
        self.SmoothNormals = None
        self.SmoothRatio = None
        self.SubdivisionAlgorithm = None
        self.SurfaceEdges = None
        self.SurfaceFaces = None
        self.SurfaceNumbers = None
        self.SwitchElementTags = None
        self.Tangents = None
        self.Tetrahedra = None
        self.ToleranceEdgeLength = None
        self.ToleranceInitialDelaunay = None
        self.Triangles = None
        self.Trihedra = None
        self.VolumeEdges = None
        self.VolumeFaces = None
        self.VolumeNumbers = None
        self.Voronoi = None
        self.ZoneDefinition = None


class ColorMesh:
    def __init__(self):
        self.Eight = None
        self.Eighteen = None
        self.Eleven = None
        self.Fifteen = None
        self.Five = None
        self.Four = None
        self.Fourteen = None
        self.Hexahedra = None
        self.Lines = None
        self.Nine = None
        self.Nineteen = None
        self.Normals = None
        self.One = None
        self.Points = None
        self.PointsSup = None
        self.Prisms = None
        self.Pyramids = None
        self.Quadrangles = None
        self.Seven = None
        self.Seventeen = None
        self.Six = None
        self.Sixteen = None
        self.Tangents = None
        self.Ten = None
        self.Tetrahedra = None
        self.Thirteen = None
        self.Three = None
        self.Triangles = None
        self.Trihedra = None
        self.Twelve = None
        self.Two = None
        self.Zero = None


class ColorGeneral:
    def __init__(self):
        self.AmbientLight = None
        self.Axes = None
        self.Background = None
        self.BackgroundGradient = None
        self.DiffuseLight = None
        self.Foreground = None
        self.SmallAxes = None
        self.SpecularLight = None
        self.Text = None


class ColorGeometry:
    def __init__(self):
        self.HighlightOne = None
        self.HighlightTwo = None
        self.HighlightZero = None
        self.Lines = None
        self.Normals = None
        self.Points = None
        self.Projection = None
        self.Selection = None
        self.Surfaces = None
        self.Tangents = None
        self.Volumes = None
